@@ -13,9 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from __future__ import print_function
-
 from oslo_config import cfg
+from oslo_config import types
 
 service_option = cfg.BoolOpt("manila",
                              default=True,
@@ -30,7 +29,7 @@ ShareGroup = [
                help="The minimum api microversion is configured to be the "
                     "value of the minimum microversion supported by Manila."),
     cfg.StrOpt("max_api_microversion",
-               default="2.51",
+               default="2.55",
                help="The maximum api microversion is configured to be the "
                     "value of the latest microversion supported by Manila."),
     cfg.StrOpt("region",
@@ -86,6 +85,15 @@ ShareGroup = [
                help="Forces access rules to be as specified on NFS scenario"
                     " tests. This can used for working around multiple "
                     "NATs between the VMs and the storage controller."),
+    cfg.StrOpt("storage_network",
+               help="Name or UUID of a neutron network that is used to access "
+                    "shared file systems over. If specified, test virtual "
+                    "machines are created with two NICs, the primary NIC is "
+                    "attached to the private project network and the "
+                    "secondary NIC is attached to the specified storage "
+                    "network. If using NFS, access control is done with the "
+                    "help of the IP address assigned to the virtual machine's "
+                    "storage network NIC."),
     cfg.ListOpt("enable_ro_access_level_for_protocols",
                 default=["nfs", ],
                 help="List of protocols to run tests with ro access level."),
@@ -167,6 +175,24 @@ ShareGroup = [
                 help="Whether to suppress errors with clean up operation "
                      "or not. There are cases when we may want to skip "
                      "such errors and catch only test errors."),
+    cfg.MultiOpt("security_service",
+                 item_type=types.Dict(),
+                 secret=True,
+                 help="This option enables specifying security service "
+                      "parameters needed to create security services "
+                      "dynamically in order to run the tempest tests. "
+                      "The configured security service must be reachable by "
+                      "the project share networks created by the tests. So, "
+                      "ideally project networks must be able to route to the "
+                      "network where the pre-existing security services has "
+                      "been deployed. The set of parameters that can be "
+                      "configured is the same used in the security service "
+                      "creation. You can repeat this option many times, and "
+                      "each entry takes the standard dict config parameters: "
+                      "security_service = "
+                      "ss_type:<ldap, kerberos or active_directory>, "
+                      "ss_dns_ip:value, ss_user:value, ss_password=value, "
+                      "ss_domain:value, ss_server:value"),
 
     # Switching ON/OFF test suites filtered by features
     cfg.BoolOpt("run_quota_tests",
@@ -231,6 +257,11 @@ ShareGroup = [
     cfg.BoolOpt("run_mount_snapshot_tests",
                 default=False,
                 help="Enable or disable mountable snapshot tests."),
+    cfg.BoolOpt("run_create_share_from_snapshot_in_another_pool_or_az_tests",
+                default=False,
+                help="Defines whether to run tests that create share from "
+                     "snapshots in another pool or az. Enable this "
+                     "option if the used driver supports it."),
 
     cfg.StrOpt("image_with_share_tools",
                default="manila-service-image-master",
@@ -260,5 +291,4 @@ ShareGroup = [
     cfg.BoolOpt("run_ipv6_tests",
                 default=False,
                 help="Enable or disable running IPv6 tests."),
-
 ]
