@@ -439,18 +439,12 @@ class MigrationOppositeDriverModesNFSTest(MigrationBase):
     def test_migration_opposite_driver_modes(self, force_host_assisted):
         self._check_migration_enabled(force_host_assisted)
 
-        share = self.create_share(self.protocol,
-                                  share_type_id=self.share_type_id)
-        share = self.shares_v2_client.get_share(share['id'])
-        share, dest_pool = self._setup_migration(share, opposite=True)
-
         if not CONF.share.multitenancy_enabled:
             # If currently configured is DHSS=False,
             # then we need it for DHSS=True
             new_share_network_id = self.provide_share_network(
                 self.shares_v2_client,
                 self.networks_client,
-                isolated_creds_client=None,
                 ignore_multitenancy_config=True,
             )
         else:
@@ -458,6 +452,12 @@ class MigrationOppositeDriverModesNFSTest(MigrationBase):
             # then we must pass None for DHSS=False
             new_share_network_id = None
 
+        share = self.create_share(self.protocol,
+                                  share_type_id=self.share_type_id,
+                                  cleanup_in_class=False)
+        share = self.shares_v2_client.get_share(share['id'])
+        share, dest_pool = self._setup_migration(share, opposite=True)
+
         old_share_network_id = share['share_network_id']
         old_share_type_id = share['share_type']
         new_share_type_id = self.new_type_opposite['share_type']['id']
